@@ -5,12 +5,14 @@ import org.broadinstitute.dropseqrna.annotation.FunctionalData;
 import org.broadinstitute.dropseqrna.annotation.FunctionalDataProcessor;
 import org.broadinstitute.dropseqrna.barnyard.Utils;
 import org.broadinstitute.dropseqrna.utils.CountChangingIteratorWrapper;
+import org.broadinstitute.dropseqrna.utils.PassFailTrackingIteratorI;
+
 import picard.annotation.LocusFunction;
 
 import java.util.*;
 
 public class GeneFunctionIteratorWrapper extends
-		CountChangingIteratorWrapper<SAMRecord> {
+		CountChangingIteratorWrapper<SAMRecord>  {
 
 	
 	private final GeneFunctionProcessor p;
@@ -28,10 +30,13 @@ public class GeneFunctionIteratorWrapper extends
 	@Override
 	public void processRecord(final SAMRecord r) {
 		List<SAMRecord> result = p.processRead(r);
+		if (result.size()==0)			
+			return;
+		
 		for (SAMRecord rr: result)
 			queueRecordForOutput(rr);
-
 	}
 
+
 	
 }
